@@ -1,3 +1,18 @@
+#include <Python.h>
+#include <math.h>
+#include <numpy/arrayobject.h>
+#include <numpy/ufuncobject.h>
+#include "healpix.h"
+#include "healpix-utils.h"
+#include "interpolation.h"
+
+/* FIXME: We need npy_set_floatstatus_invalid(), but unlike most of the Numpy
+ * C API it is only available on some platforms if you explicitly link against
+ * Numpy, which is not typically done for building C extensions. This bundled
+ * header file contains a static definition of _npy_set_floatstatus_invalid().
+ */
+#include "ieee754.h"
+
 /* FIXME:
  * The Numpy C-API defines PyArrayDescr_Type as:
  *
@@ -14,23 +29,11 @@
  *
  * Drop this when supporting gcc < 10 becomes irrelevant.
  */
+#ifndef PYPY_VERSION
 struct _typeobject {
     int _placeholder;
 };
-
-#include <Python.h>
-#include <numpy/arrayobject.h>
-#include <numpy/ufuncobject.h>
-#include "healpix.h"
-#include "healpix-utils.h"
-#include "interpolation.h"
-
-/* FIXME: We need npy_set_floatstatus_invalid(), but unlike most of the Numpy
- * C API it is only available on some platforms if you explicitly link against
- * Numpy, which is not typically done for building C extensions. This bundled
- * header file contains a static definition of _npy_set_floatstatus_invalid().
- */
-#include "ieee754.h"
+#endif
 
 
 #define INVALID_INDEX (-1)
@@ -67,7 +70,7 @@ static int pixel_nside_valid(int64_t pixel, int nside)
 
 
 static void healpix_to_lonlat_loop(
-    char **args, npy_intp *dimensions, npy_intp *steps, void *data)
+    char **args, const npy_intp *dimensions, const npy_intp *steps, void *data)
 {
     order_funcs *funcs = data;
     npy_intp i, n = dimensions[0];
@@ -97,7 +100,7 @@ static void healpix_to_lonlat_loop(
 
 
 static void lonlat_to_healpix_loop(
-    char **args, npy_intp *dimensions, npy_intp *steps, void *data)
+    char **args, const npy_intp *dimensions, const npy_intp *steps, void *data)
 {
     order_funcs *funcs = data;
     npy_intp i, n = dimensions[0];
@@ -112,7 +115,8 @@ static void lonlat_to_healpix_loop(
         double  *dy    =  (double *)  &args[5][i * steps[5]];
         int64_t xy = INVALID_INDEX;
 
-        xy = radec_to_healpixlf(lon, lat, nside, dx, dy);
+        if (isfinite(lon) && isfinite(lat))
+            xy = radec_to_healpixlf(lon, lat, nside, dx, dy);
         if (xy >= 0)
             *pixel = funcs->xy_to_order(xy, nside);
         else {
@@ -125,7 +129,7 @@ static void lonlat_to_healpix_loop(
 
 
 static void healpix_to_xyz_loop(
-    char **args, npy_intp *dimensions, npy_intp *steps, void *data)
+    char **args, const npy_intp *dimensions, const npy_intp *steps, void *data)
 {
     order_funcs *funcs = data;
     npy_intp i, n = dimensions[0];
@@ -156,7 +160,7 @@ static void healpix_to_xyz_loop(
 
 
 static void xyz_to_healpix_loop(
-    char **args, npy_intp *dimensions, npy_intp *steps, void *data)
+    char **args, const npy_intp *dimensions, const npy_intp *steps, void *data)
 {
     order_funcs *funcs = data;
     npy_intp i, n = dimensions[0];
@@ -172,13 +176,15 @@ static void xyz_to_healpix_loop(
         double  *dy    =  (double *)  &args[6][i * steps[6]];
         int64_t xy = INVALID_INDEX;
 
-        /* xyztohealpixlf expects a unit vector */
-        double norm = sqrt(x*x + y*y + z*z);
-        x /= norm;
-        y /= norm;
-        z /= norm;
+        if (isfinite(x) && isfinite(y) && isfinite(z)) {
+            /* xyztohealpixlf expects a unit vector */
+            double norm = sqrt(x*x + y*y + z*z);
+            x /= norm;
+            y /= norm;
+            z /= norm;
 
-        xy = xyztohealpixlf(x, y, z, nside, dx, dy);
+            xy = xyztohealpixlf(x, y, z, nside, dx, dy);
+        }
         if (xy >= 0)
             *pixel = funcs->xy_to_order(xy, nside);
         else {
@@ -191,7 +197,7 @@ static void xyz_to_healpix_loop(
 
 
 static void nested_to_ring_loop(
-    char **args, npy_intp *dimensions, npy_intp *steps, void *data)
+    char **args, const npy_intp *dimensions, const npy_intp *steps, void *data)
 {
     npy_intp i, n = dimensions[0];
 
@@ -215,7 +221,7 @@ static void nested_to_ring_loop(
 
 
 static void ring_to_nested_loop(
-    char **args, npy_intp *dimensions, npy_intp *steps, void *data)
+    char **args, const npy_intp *dimensions, const npy_intp *steps, void *data)
 {
     npy_intp i, n = dimensions[0];
 
@@ -239,7 +245,7 @@ static void ring_to_nested_loop(
 
 
 static void bilinear_interpolation_weights_loop(
-    char **args, npy_intp *dimensions, npy_intp *steps, void *data)
+    char **args, const npy_intp *dimensions, const npy_intp *steps, void *data)
 {
     npy_intp i, n = dimensions[0];
 
@@ -263,7 +269,7 @@ static void bilinear_interpolation_weights_loop(
 
 
 static void neighbours_loop(
-    char **args, npy_intp *dimensions, npy_intp *steps, void *data)
+    char **args, const npy_intp *dimensions, const npy_intp *steps, void *data)
 {
     order_funcs *funcs = data;
     npy_intp i, n = dimensions[0];
